/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef _INPUTWINDOWPROXY_H_
#define _INPUTWINDOWPROXY_H_

#include <memory>

#include <fcitx-utils/dbus/bus.h>
#include <fcitx-utils/misc.h>
#include <fcitx-utils/utf8.h>
#include <fcitx/candidatelist.h>
#include <fcitx/inputcontext.h>
#include <fcitx/inputpanel.h>
#include <fcitx/instance.h>
#include <fcitx/text.h>

class InputWindowBackend;

class InputWindowProxy {
public:
    InputWindowProxy(fcitx::Instance *instance, fcitx::dbus::Bus *bus);
    ~InputWindowProxy();

    void updateInputPanel(fcitx::InputContext *inputContext);
    void selectCandidate(int index);
    void prevPage();
    void nextPage();

private:
    void initInputWindowBackend();
    void updateCursorPosition(fcitx::InputContext *inputContext);
    void updatePreeditArea(fcitx::InputContext *inputContext);
    void updateCandidateArea(fcitx::InputContext *inputContext);

    void notifyCursorPosition(int x, int y, int width, int height);
    void notifyPreeditText(const std::string &preeditString);
    void notifyPreeditCaret(int utf8Cursor);
    void notifyCandidateArea(const std::vector<std::string> &candidateTextList,
                             bool hasPrev, bool hasNext, int pageIndex,
                             int highlightIndex);

private:
    fcitx::Instance *instance_;
    fcitx::dbus::Bus *bus_;
    std::shared_ptr<InputWindowBackend> inputWindowBackend_ = nullptr;
};
#endif