/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef _OKINPUTMETHODENGINE_H_
#define _OKINPUTMETHODENGINE_H_

#include <memory>

#include <fcitx/action.h>
#include <fcitx/addonfactory.h>
#include <fcitx/addoninstance.h>
#include <fcitx/addonmanager.h>
#include <fcitx/inputcontext.h>
#include <fcitx/inputmethodengine.h>
#include <fcitx/inputpanel.h>
#include <fcitx/instance.h>
#include <fcitx/userinterfacemanager.h>

#include "../imestatustypes/inputmethodstatustypes.h"
#include "okpinyin/okpinyin.h"

class OKInputMethodEngine : public fcitx::InputMethodEngineV3 {
public:
    explicit OKInputMethodEngine(fcitx::Instance *instance = nullptr);
    ~OKInputMethodEngine() override;

    void activate(const fcitx::InputMethodEntry &entry,
                  fcitx::InputContextEvent &event) override;
    void deactivate(const fcitx::InputMethodEntry &entry,
                    fcitx::InputContextEvent &event) override;
    void keyEvent(const fcitx::InputMethodEntry &entry,
                  fcitx::KeyEvent &keyEvent) override;
    void reloadConfig() override;
    void reset(const fcitx::InputMethodEntry &entry,
               fcitx::InputContextEvent &event) override;
    void save() override;
    std::string subMode(const fcitx::InputMethodEntry &entry,
                        fcitx::InputContext &inputContext) override;
    void invokeActionImpl(const fcitx::InputMethodEntry &entry,
                          fcitx::InvokeActionEvent &event) override;
    const fcitx::Configuration *getConfig() const override;
    void setConfig(const fcitx::RawConfig &config) override;
    void setSubConfig(const std::string &path,
                      const fcitx::RawConfig &config) override;

    const InputMethodStatus::StatusBarAction &language();
    const InputMethodStatus::StatusBarAction &chttransInfo();
    const InputMethodStatus::StatusBarAction &charWidthInfo();
    const InputMethodStatus::StatusBarAction &punctuationInfo();
    const InputMethodStatus::StatusBarAction &inputSchemeInfo();
    bool switchStatusActionBar(int actionType) const;

private:
    fcitx::Instance *instance_;
    std::unique_ptr<fcitx::PinyinEngine> pinyinEngine_ = nullptr;

    fcitx::Action *chttrans_;
    fcitx::Action *charWidth_;
    fcitx::Action *punctuation_;
    InputMethodStatus::StatusBarAction languageAction_;
    InputMethodStatus::StatusBarAction chttransAction_;
    InputMethodStatus::StatusBarAction charWidthAction_;
    InputMethodStatus::StatusBarAction punctuationAction_;
    InputMethodStatus::StatusBarAction inputSchemeAction_;
};

#endif