/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef _STATUSBARPROXY_H_
#define _STATUSBARPROXY_H_

#include <fcitx-utils/dbus/bus.h>
#include <fcitx-utils/dbus/objectvtable.h>

#include "enginemanager.h"

class StatusBarBackend;
class StatusBarProxy {
public:
    StatusBarProxy(EngineManager *engineManager, fcitx::dbus::Bus *bus);
    ~StatusBarProxy();

    void imeActivated();
    void imeDeactivated();
    void updateStatusBar();

    std::vector<InputMethodStatus::StatusBarAction> &currentStatusBarActions();
    bool setStatusBarAction(int index) const;

private:
    void initStatusBarBackend();
    void notifyIMEActivated();
    void notifyIMEDeactivated();
    void notifyStatusBarArea();
    void updateStatusBarActions();

private:
    EngineManager *engineManager_;
    fcitx::dbus::Bus *bus_;
    std::shared_ptr<StatusBarBackend> statusBarBackend_ = nullptr;
    std::vector<InputMethodStatus::StatusBarAction> actions_;
    bool currentIMEActivated = false;
};
#endif