/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "uiproxy.h"

UIProxy::UIProxy(fcitx::Instance *instance, EngineManager *engineManager)
    : instance_(instance), engineManager_(engineManager),
      bus_(dbus()->call<fcitx::IDBusModule::bus>()) {
    initProxy();
    connectSignal();
}

UIProxy::~UIProxy() = default;

void UIProxy::initProxy() {
    statusBarProxy_ = std::make_unique<StatusBarProxy>(engineManager_, bus_);
    inputWindowProxy_ = std::make_unique<InputWindowProxy>(instance_, bus_);
}

void UIProxy::connectSignal() {
    eventHandlers_.emplace_back(instance_->watchEvent(
        fcitx::EventType::InputContextUpdateUI,
        fcitx::EventWatcherPhase::Default, [this](fcitx::Event &event) {
            if (static_cast<fcitx::InputContextUpdateUIEvent &>(event)
                    .component() == fcitx::UserInterfaceComponent::StatusArea) {
                if (activate_ == false) {
                    return;
                }
                updateStatusAction();
            }
        }));

    eventHandlers_.emplace_back(instance_->watchEvent(
        fcitx::EventType::InputContextUpdateUI,
        fcitx::EventWatcherPhase::PreInputMethod, [this](fcitx::Event &event) {
            if (static_cast<fcitx::InputContextUpdateUIEvent &>(event)
                    .component() == fcitx::UserInterfaceComponent::InputPanel) {
                if (activate_ == false) {
                    return;
                }

                auto *ic = instance_->mostRecentInputContext();
                if (ic == nullptr) {
                    return;
                }

                ic->inputPanel().setCustomInputPanelCallback(
                    [this](fcitx::InputContext *inputContext) {
                        inputWindowProxy_->updateInputPanel(inputContext);
                    });
            }
        }));
}

void UIProxy::notifyIMEActivate() {
    activate_ = true;
    statusBarProxy_->imeActivated();
}

void UIProxy::notifyIMEDeactivate() {
    activate_ = false;
    statusBarProxy_->imeDeactivated();
}

void UIProxy::updateStatusAction() { statusBarProxy_->updateStatusBar(); }

void UIProxy::updateInputPanel(fcitx::InputContext *inputcontext) {
    inputWindowProxy_->updateInputPanel(inputcontext);
}