/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef _UIPROXY_H_
#define _UIPROXY_H_

#include <memory>

#include <dbus_public.h>
#include <fcitx-utils/dbus/bus.h>
#include <fcitx-utils/dbus/servicewatcher.h>
#include <fcitx-utils/event.h>

#include "enginemanager.h"
#include "inputwindowproxy.h"
#include "statusbarproxy.h"

class UIProxy {
public:
    UIProxy(fcitx::Instance *instance, EngineManager *engineManager);
    ~UIProxy();
    void notifyIMEActivate();
    void notifyIMEDeactivate();
    void updateStatusAction();
    void updateInputPanel(fcitx::InputContext *inputcontext);

private:
    void initProxy();
    void connectSignal();

private:
    FCITX_ADDON_DEPENDENCY_LOADER(dbus, instance_->addonManager());

private:
    fcitx::Instance *instance_;
    EngineManager *engineManager_;
    fcitx::dbus::Bus *bus_ = nullptr;
    std::unique_ptr<StatusBarProxy> statusBarProxy_ = nullptr;
    std::unique_ptr<InputWindowProxy> inputWindowProxy_ = nullptr;
    std::vector<std::unique_ptr<fcitx::HandlerTableEntry<fcitx::EventHandler>>>
        eventHandlers_;
    bool activate_ = false;
};
#endif