/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef INPUTWINDOWMANAGER_H
#define INPUTWINDOWMANAGER_H

#include <functional>
#include <memory>

#include <QObject>
#include <QRect>

#include "inputwindowview.h"

class InputWindowModel;

class InputWindowManager : public QObject {
    Q_OBJECT
public:
    using SelectCandidateCallback = std::function<void(int)>;
    using PrevPageCallback = std::function<void()>;
    using NextPageCallback = std::function<void()>;

public:
    explicit InputWindowManager(QObject *parent = nullptr);
    ~InputWindowManager();

    void updateCursorRect(const QRect &cursorRect);
    void updatePreeditText(const QString &preeditText);
    void updatePreeditCaret(int caretIndex);
    void updateCandidateArea(const QStringList &candidateTextList, bool hasPrev,
                             bool hasNext, int pageIndex,
                             int localHighlightIdx);
    void showInputWindow();
    void hideInputWindow();

    void setSelectCandidateCallback(SelectCandidateCallback callback);
    void setPrevPageCallback(PrevPageCallback callback);
    void setNextPageCallback(NextPageCallback callback);

    Q_INVOKABLE void selectCandidate(int localIdx);
    Q_INVOKABLE void prevPage();
    Q_INVOKABLE void nextPage();

private:
    SelectCandidateCallback m_selectCandidateCallback = nullptr;
    PrevPageCallback m_prevPageCallback = nullptr;
    NextPageCallback m_nextPageCallback = nullptr;

    InputWindowModel *m_inputWindowModel = nullptr;
    std::unique_ptr<InputWindowView> m_inputWindowView = nullptr;
};

#endif // INPUTWINDOWMANAGER_H
