/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef DBUSCHANNEL_H
#define DBUSCHANNEL_H

#include <memory>

#include <QDBusInterface>
#include <QDBusServiceWatcher>
#include <QObject>
#include <QString>

#include "ipctypes.h"

class InputWindowManager;
class StatusBarManager;

class DBusChannel : public QObject {
    Q_OBJECT
public:
    explicit DBusChannel(InputWindowManager &inputWindowManager,
                         StatusBarManager &statusBarManager,
                         QObject *parent = nullptr);
    ~DBusChannel() = default;

public slots:
    void InputMethodEngineActivated();
    void InputMethodEngineDeactivated();
    void UpdateCursorRect(int x, int y, int width, int height);
    void UpdatePreeditText(const QString &preeditText);
    void UpdatePreeditCaret(int caretIndex);
    void UpdateCandidateArea(const QStringList &candidateTextList, bool hasPrev,
                             bool hasNext, int pageIndex,
                             int localHighlightIdx);
    void
    UpdateStatusBarArea(QList<InputMethodStatus::StatusBarAction> actionList);

private slots:
    void inputMethodBackendServiceRegistered(const QString &serviceName);
    void inputMethodBackendServiceUnregistered(const QString &serviceName);

private:
    bool startDBusService();
    void setCallback();
    void initInputMethodBackendInterface();
    void initInputMethodBackendServiceWatcher();
    void closeApp();

    void selectCandidate(int localIdx);
    void prevPage();
    void nextPage();

    void switchStatusBarAction(int index);
    void requestStatusBarActions();

private:
    QString m_serviceName = "org.ukui.okinputmethod";
    QString m_servicePath = "/org/ukui/okinputmethod";
    QString m_serviceInterface = "org.ukui.okinputmethod";

    QString m_inputMethodBackendServiceName = "org.ukui.inputmethodbackend";
    QString m_inputWindowPath = "/inputwindowproxy";
    QString m_inputWindowInterface = "org.ukui.inputmethod.inputwindowproxy";
    QString m_statusBarPath = "/statusbarproxy";
    QString m_statusBarInterface = "org.ukui.inputmethod.statusbarproxy";

    std::unique_ptr<QDBusServiceWatcher> m_inputMethodBackendServiceWatcher =
        nullptr;
    std::unique_ptr<QDBusInterface> m_inputWindowBackendInterface = nullptr;
    std::unique_ptr<QDBusInterface> m_statusBarBackendInterface = nullptr;

    InputWindowManager &m_inputWindowManager;
    StatusBarManager &m_statusBarManager;
};

#endif // DBUSCHANNEL_H
