/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

import QtQuick 2.0

Rectangle {
    width: candidateListView.width + inputWindow.candidateLeftMargin*2
    height: getCandidateListViewHeight()
    radius: inputWindow.inputWindowRadius
    color: "transparent"
    anchors.left: parent.left
    anchors.bottom: parent.bottom
    anchors.bottomMargin: inputWindow.candidateBottomMargin
    antialiasing: true

    function getCandidateListViewHeight() {
        return candidateListView.height >= inputWindow.candidateDefaultHeight
                ? candidateListView.height : inputWindow.candidateDefaultHeight
    }

    Rectangle {
        anchors.fill: parent
        color: parent.color
        radius: parent.radius
        antialiasing: true
    }

    ListView {
        id: candidateListView
        width: contentItem.childrenRect.width
        height: contentItem.childrenRect.height
        orientation: ListView.Horizontal
        anchors.centerIn: parent
        model: inputWindowModel.candidateModel
        spacing: inputWindow.candidateSpace
        delegate: Item {
            width: inputWindow.orderNumberLeftMargin * 2
                   + orderNumber.width
                   + inputWindow.orderNumberAndCandidateSpace
                   + candidate.width
            height: candidate.height
            anchors.verticalCenter: parent.verticalCenter

            property string candidateState: "Normal"            
            state: modelData.highlighted ? "Press" : candidateState
            states: [
                State {
                    name: "Normal"
                    PropertyChanges {
                        target: candidateListBack
                        color: "transparent"
                    }
                    PropertyChanges {
                        target: orderNumber
                        color: inputWindow.orderNumberDefaultColor
                        opacity: inputWindow.orderNumberDefaultOpacity
                    }
                    PropertyChanges {
                        target: candidate
                        color: inputWindow.candidateDefaultFontColor
                    }
                },
                State {
                    name: "Hover"
                    PropertyChanges {
                        target: candidateListBack
                        color: inputWindow.candidateBackHoverColor
                        opacity: inputWindow.candidateBackHoverOpacity
                    }
                    PropertyChanges {
                        target: orderNumber
                        color: inputWindow.orderNumberDefaultColor
                        opacity: inputWindow.orderNumberDefaultOpacity
                    }
                    PropertyChanges {
                        target: candidate
                        color: inputWindow.candidateDefaultFontColor
                    }
                },
                State {
                    name: "Press"
                    PropertyChanges {
                        target: candidateListBack
                        color: inputWindow.candidateBackPressColor
                        opacity: 1
                    }
                    PropertyChanges {
                        target: orderNumber
                        color: inputWindow.orderNumberFontPressColor
                        opacity: 1
                    }
                    PropertyChanges {
                        target: candidate
                        color: inputWindow.candidateFontPressColor
                    }
                }
            ]

            Rectangle {
                id: candidateListBack
                anchors.fill: parent
                radius: inputWindow.candidateRadius
            }

            Text {
                id: orderNumber
                text: modelData.index
                font.pointSize: inputWindow.orderNumberFontSize
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: parent.left
                anchors.leftMargin: inputWindow.orderNumberLeftMargin
            }
            Text {
                id: candidate
                text: modelData.text
                font.pointSize: inputWindow.candidateFontSize
                anchors.left: orderNumber.right
                anchors.leftMargin: inputWindow.orderNumberAndCandidateSpace
                anchors.verticalCenter: parent.verticalCenter
            }

            MouseArea {
                anchors.fill: parent
                hoverEnabled: true
                onEntered: {
                    candidateState = "Hover"
                }
                onExited: {
                    candidateState = "Normal"
                }
                onPressed: {
                    candidateState = "Press"
                }
                onClicked: {
                    inputWindowManager.selectCandidate(index)
                }
            }
        }
    }
}

