/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import org.ukui.qqc2style.private 1.0 as StylePrivate

Rectangle {
    id: inputWindow
    width: getInputWindowWidth()
    height: preedit.height+
            candidateList.height+
            candidateBottomMargin*2
    radius: inputWindowRadius
    antialiasing: true
    color: "transparent"

    readonly property int candidateSpace: 4
    readonly property int orderNumberAndCandidateSpace: 4
    readonly property int candidateLeftMargin: 8
    readonly property int candidateBottomMargin: 8
    readonly property int preeditLeftMargin: 8
    readonly property int preeditTopMargin: 4
    readonly property int orderNumberLeftMargin: 4
    readonly property int pageButtonWidth: 16
    readonly property int pageButtonHeight: 24
    readonly property int nextPageButtonRightMagin: 8
    readonly property int candidateDefaultHeight: 28
    readonly property int borderWidth: 2
    readonly property int preeditHeight: 20

    readonly property color inputWindowBackColor: StylePrivate.StyleHelper.basecolorrole
    readonly property color inputWindowBackHightlightColor: StylePrivate.StyleHelper.highlightcolorrole
    readonly property color candidateBackPressColor: StylePrivate.StyleHelper.highlightcolorrole
    readonly property color candidateBackHoverColor: StylePrivate.StyleHelper.brighttextcolorrole
    readonly property color candidateFontPressColor: "#FFFFFF"
    readonly property color candidateDefaultFontColor: StylePrivate.StyleHelper.windowtextcolorrole
    readonly property color orderNumberFontPressColor: "#FFFFFF"
    readonly property color orderNumberDefaultColor: StylePrivate.StyleHelper.windowtextcolorrole
    readonly property color turnPageButtonBackColor: StylePrivate.StyleHelper.brighttextcolorrole
    readonly property color borderColor: StylePrivate.StyleHelper.buttoncolorrole

    readonly property real inputWindowBackHightlightOpacity: 0.05
    readonly property real candidateBackHoverOpacity: 0.15
    readonly property real orderNumberDefaultOpacity: 0.3
    readonly property real turnPageButtonBackHoverOpacity: 0.15
    readonly property real turnPageButtonBackPressOpacity: 0.3

    readonly property int inputWindowRadius: 8
    readonly property int candidateRadius: 4

    readonly property int preeditFontSize: 12
    readonly property int orderNumberFontSize: 10
    readonly property int candidateFontSize: 14

    readonly property string preeditText: inputWindowModel.preeditText
    readonly property int cursorPosition: inputWindowModel.preeditCaret
    readonly property string nextPageButtonImageSource: "ukui-end-symbolic"
    readonly property string prevPageButtonImageSource: "ukui-start-symbolic"
    readonly property bool hasPrev: inputWindowModel.hasPrev
    readonly property bool hasNext: inputWindowModel.hasNext

    function getInputWindowWidth() {
        var tmpWidth = candidateList.width + prevPageButton.width
                + nextPageButton.width + candidateSpace
        return preedit.width >= tmpWidth ? preedit.width : tmpWidth
    }

    Rectangle {
        anchors.fill: parent
        color: inputWindowBackColor
        radius: parent.radius
        antialiasing: true
        border.color: borderColor
        border.width: borderWidth
    }
    Rectangle {
        anchors.fill: parent
        color: inputWindowBackHightlightColor
        opacity: inputWindowBackHightlightOpacity
        radius: parent.radius
    }

    Preedit {
        id: preedit
    }

    CandidateList {
        id: candidateList
    }

    TurnPageButton {
        id: prevPageButton
        anchors.right: nextPageButton.left
        anchors.verticalCenter: candidateList.verticalCenter
        imageSource: inputWindow.prevPageButtonImageSource
        hasPage: inputWindow.hasPrev
        mouseArea.onClicked: {
            if (hasPage) {
                inputWindowManager.prevPage()
            }
        }
    }

    TurnPageButton {
        id: nextPageButton
        anchors.right: parent.right
        anchors.rightMargin: inputWindow.nextPageButtonRightMagin
        anchors.verticalCenter: candidateList.verticalCenter
        imageSource: inputWindow.nextPageButtonImageSource
        hasPage: inputWindow.hasNext
        mouseArea.onClicked: {
            if (hasPage) {
                inputWindowManager.nextPage()
            }
        }
    }
}
