/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import org.ukui.qqc2style.private 1.0 as StylePrivate

Rectangle {
    id: statusBar
    width: dragBar.width +
           statusBarButton.width +
           statusBarButtonRightMargin * 2
    height: statusBarHeight
    radius: statusBarRadius
    antialiasing: true
    color: "transparent"

    readonly property int iconBackWidth: 24
    readonly property int iconBackHeight: 24
    readonly property int iconWidth: 16
    readonly property int iconHeight: 16
    readonly property int statusBarHeight: 32
    readonly property int dragBarWidth: 32
    readonly property int dragBarHeight: 36
    readonly property int buttonSpacing: 4
    readonly property int statusBarButtonRightMargin: 5
    readonly property int borderWidth: 2

    readonly property color iconBackPressColor: StylePrivate.StyleHelper.highlightcolorrole
    readonly property color iconBackHoverColor: StylePrivate.StyleHelper.highlightcolorrole
    readonly property color statusBarBackColor: StylePrivate.StyleHelper.basecolorrole
    readonly property color statusBarBackHightlightColor: StylePrivate.StyleHelper.highlightcolorrole
    readonly property color borderColor: StylePrivate.StyleHelper.buttoncolorrole

    readonly property real iconBackHoverOpacity: 0.15
    readonly property real statusBarBackHightlightOpacity: 0.05

    readonly property int statusBarRadius: 8
    readonly property int buttonRadius: 4

    Rectangle {
        anchors.fill: parent
        color: statusBar.statusBarBackColor
        radius: parent.radius
        antialiasing: true
        border.color: borderColor
        border.width: borderWidth
    }
    Rectangle {
        anchors.fill: parent
        color: statusBar.statusBarBackHightlightColor
        opacity: statusBar.statusBarBackHightlightOpacity
        radius: parent.radius
    }

    DragBar {
        id: dragBar
    }

    StatusBarButton {
        id: statusBarButton
    }
}
