/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

import QtQuick 2.0
import org.ukui.qqc2style.private 1.0 as StylePrivate
import ToolTip 2.0

Row {
    spacing: buttonSpacing
    anchors.right: parent.right
    anchors.rightMargin: statusBar.statusBarButtonRightMargin
    anchors.verticalCenter: parent.verticalCenter
    Repeater {
        model: statusBarModel
        delegate: Rectangle {
            id: statusBarButtonBack
            width: statusBar.iconBackWidth
            height: statusBar.iconBackHeight
            radius: statusBar.buttonRadius
            color: "transparent"

            Rectangle {
                id: iconBack
                visible: modelData.isClickable
                width: parent.width
                height: parent.height
                radius: parent.radius
                color: statusBar.statusBarBackColor
                opacity: 0
            }
            StylePrivate.KyIcon {
                id: icon
                width: statusBar.iconWidth
                height: statusBar.iconHeight
                opacity: modelData.isClickable ? 1 : 0.5
                iconName: modelData.iconName
                anchors.centerIn: parent
            }

            MouseArea {
                anchors.fill: parent
                hoverEnabled: true
                onEntered: {
                    iconBack.color = statusBar.iconBackHoverColor
                    iconBack.opacity = statusBar.iconBackHoverOpacity
                }
                onPressed: {
                    iconBack.color = statusBar.iconBackPressColor
                    iconBack.opacity = 1
                    if (modelData.isClickable) {
                        icon.hover = true
                    }
                }
                onReleased: {
                    iconBack.color = statusBar.statusBarBackColor
                    iconBack.opacity = 0
                    icon.hover = false
                }
                onExited: {
                    iconBack.color = statusBar.statusBarBackColor
                    iconBack.opacity = 0
                    icon.hover = false
                }
                onClicked: {
                    statusBarManager.switchStatusBarAction(index)
                }

                ToolTipArea {
                    id: toolTipArea
                    anchors.fill: parent
                    toolTipText: modelData.toolTip
                    anchors.centerIn: parent
                }
            }
        }
    }
}
