/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef STATUSBARMANAGER_H
#define STATUSBARMANAGER_H

#include <functional>

#include <QObject>

#include "statusbarmodel.h"
#include "statusbarsettings.h"

namespace InputMethodStatus {
struct StatusBarAction;
}

class StatusBarView;

class StatusBarManager : public QObject {
    Q_OBJECT
public:
    using SwitchStatusBarActionCallback = std::function<void(int)>;
    using RequestStatusBarActionCallback = std::function<void()>;

public:
    explicit StatusBarManager(QObject *parent = nullptr);

    void
    updateStatusBarArea(QList<InputMethodStatus::StatusBarAction> &actionList);
    void showStatusBar();
    void hideStatusBar();

    void
    setSwitchStatusBarActionCallback(SwitchStatusBarActionCallback callback);
    void
    setRequestStatusBarActionsCallback(RequestStatusBarActionCallback callback);

    Q_INVOKABLE void switchStatusBarAction(int index);

private:
    QPoint getPersistentStatusBarPos();
    // 根据全局坐标和单个屏幕的左上边缘占比重新计算状态栏的位置
    QPoint recalculateStatusBarPos(const QPoint &globalPos, float leftRatio,
                                   float topRatio);
    void
    sortStatusBarActions(QList<InputMethodStatus::StatusBarAction> &actionList);

private:
    SwitchStatusBarActionCallback m_switchStatusBarActionCallback = nullptr;
    RequestStatusBarActionCallback m_requestStatusBarActionsCallback = nullptr;

    // 当前版本先固定排序，后面要增加自定义排序的功能
    QList<int> actionsSequence = {0, 1, 2, 4, 3};

    StatusBarSettings m_statusBarSettings;
    StatusBarModel m_statusBarModel;
    StatusBarView *m_statusBarView = nullptr;
};

#endif // STATUSBARMANAGER_H
