/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef STATUSBARMODEL_H
#define STATUSBARMODEL_H

#include <memory>

#include <QAbstractListModel>

#include "inputmethodstatustypes.h"
#include "statusbaritem.h"

class StatusBarModel : public QAbstractListModel {
    Q_OBJECT
public:
    enum ModelDataRoles {
        ModelDataRole = Qt::UserRole + 1,
    };

    explicit StatusBarModel(QObject *parent = nullptr);
    virtual ~StatusBarModel() override = default;

    QHash<int, QByteArray> roleNames() const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    QVariant data(const QModelIndex &index,
                  int role = Qt::DisplayRole) const override;

    void updateStatusBarButton(
        const QList<InputMethodStatus::StatusBarAction> &actionList);

private:
    void clearModelData();
    void
    addModelData(const QList<InputMethodStatus::StatusBarAction> &actionList);

    std::shared_ptr<StatusBarItem>
    createStatusBarButton(InputMethodStatus::StatusBarAction action);

    void
    setStatusBarToolTip(std::shared_ptr<StatusBarItem> item,
                        InputMethodStatus::StatusBarAction statusBarAction);
    void setStatusBarButtonProperty(
        std::shared_ptr<StatusBarItem> item,
        InputMethodStatus::StatusBarAction statusBarAction);
    void setLanguageToolTip(std::shared_ptr<StatusBarItem> item,
                            const QString &toolTip);
    void setCharWidthToolTip(std::shared_ptr<StatusBarItem> item,
                             const QString &toolTip);
    void setPunctuationToolTip(std::shared_ptr<StatusBarItem> item,
                               const QString &toolTip);
    void setChttransToolTip(std::shared_ptr<StatusBarItem> item,
                            const QString &toolTip);
    void setInputSchemeToolTip(std::shared_ptr<StatusBarItem> item,
                               const QString &toolTip);

    void setLanguageIcon(std::shared_ptr<StatusBarItem> item,
                         InputMethodStatus::Language language);
    void setCharWidthIcon(std::shared_ptr<StatusBarItem> item,
                          InputMethodStatus::CharWidth charWidth);
    void setPunctuationIcon(std::shared_ptr<StatusBarItem> item,
                            InputMethodStatus::Punctuation punctuation);
    void setChttransIcon(std::shared_ptr<StatusBarItem> item,
                         InputMethodStatus::Chttrans chttrans);
    void setInputSchemeIcon(std::shared_ptr<StatusBarItem> item,
                            InputMethodStatus::PinyinScheme inputScheme);

private:
    QList<std::shared_ptr<StatusBarItem>> m_statusBarButtonList;
};

Q_DECLARE_METATYPE(std::shared_ptr<StatusBarItem>)

#endif // STATUSBARMODEL_H
