/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef STATUSBARVIEW_H
#define STATUSBARVIEW_H

#include <QObject>
#include <QQuickView>

#include "geometrymanager.h"

class StatusBarView : public QObject {
    Q_OBJECT
public:
    StatusBarView(QObject &model, QObject &manager, QPoint statusBarPos,
                  QObject *parent = nullptr);

    QPoint statusBarPosition() const;
    void show();
    void hide();
    Q_INVOKABLE void moveBy(int offsetx, int offsety);
    Q_INVOKABLE void startDrag();
    Q_INVOKABLE void endDrag();

private:
    void setViewPosition(QPoint viewPosition);
    void initConnections();
    void initView(QPoint statusBarPos);
    QPoint getPosition();

private:
    QObject &m_model;
    QObject &m_manager;

    QQuickView m_view;
    GeometryManager m_geometryManager;
};

#endif // STATUSBARVIEW_H
