/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef TOOLTIP_H
#define TOOLTIP_H

#include <QQuickItem>
#include <QQuickView>
#include <QTimer>

#include "geometrymanager.h"

class ToolTip : public QQuickItem {
    Q_OBJECT
public:
    Q_PROPERTY(QString toolTipText READ toolTipText WRITE setToolTipText NOTIFY
                   toolTipTextChanged)

    ToolTip(QQuickItem *parent = nullptr);
    ~ToolTip() override = default;

    QString toolTipText() const;
    void setToolTipText(const QString &text);

    Q_INVOKABLE void hideToolTip();

protected:
    void hoverEnterEvent(QHoverEvent *event) override;
    void hoverLeaveEvent(QHoverEvent *event) override;
    void mousePressEvent(QMouseEvent *event) override;

signals:
    void toolTipTextChanged();
    void backColorChanged();

private:
    void showToolTip();
    void setViewGeometry(QRect viewGeometry);
    QQuickItem *toolTipItem();
    void initMouseEvent();
    void initTimer();
    void initView();
    void startTimeoutTimer();
    void startShowTimer(int msec);
    void startHideTimer(int msec);
    QPoint getPosition();
    QRect getScreenGeometryOfPoint(QPoint point);
    int getX(int screenWidth, int toolTipWidth);
    int getY(int toolTipHeight);
    int parentItemX();
    int parentItemY();

private:
    const int statusBarAndTooltipSpacing = 6;

    QString m_toolTipText;
    int m_mouseX;
    QTimer m_timer;
    QQuickView m_view;
    GeometryManager m_geometryManager;
};

#endif // TOOLTIP_H
