/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#include "inputwindowgeometrymanager.h"

#include <QDebug>

InputWindowGeometryManager::InputWindowGeometryManager(GeometryManager *parent)
    : GeometryManager(parent) {}

void InputWindowGeometryManager::setCursorPosition(const QRect &pos) {
    m_cursorPosition = pos;
}

void InputWindowGeometryManager::updateValidPosition() {
    QRect newGeometry = m_geometry.translated(m_cursorPosition.width(),
                                              m_cursorPosition.height());
    QRect currentScreenGeometry =
        getScreenGeometryOfPoint(newGeometry.topLeft());

    // X-coordinate
    if (newGeometry.right() > currentScreenGeometry.right() &&
        newGeometry.width() < currentScreenGeometry.width()) {
        newGeometry.moveRight(currentScreenGeometry.right());
    }
    // Y-coordinate
    if (newGeometry.bottom() > currentScreenGeometry.bottom()) {
        newGeometry.moveBottom(m_cursorPosition.top());
    }
    m_geometry = newGeometry;
}