/*
 * Copyright 2023 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef STATUSBARSETTINGS_H
#define STATUSBARSETTINGS_H

#include <QPoint>
#include <QStandardPaths>

class StatusBarSettings {

public:
    struct StatusBarPositionData {
        int globalX;
        int globalY;
        float regionalLeftRatio;
        float regionalTopRatio;
    };
    StatusBarSettings();

    StatusBarPositionData getStatusBarPos() const;
    void saveStatusBarPos(int globalX, int globalY);

private:
    bool homeConfigFileExists() const;
    bool createHomeConfigFile();
    bool statusBarPosDataExists() const;
    void initStatusBarDefaultPos();
    void saveStatusBarPositionData(const StatusBarPositionData &pos);

private:
    const QString m_configFilePath =
        "/usr/share/ok-input-method/ok-input-method.conf";
    const QString m_homeConfigFileDir =
        QStandardPaths::writableLocation(
            QStandardPaths::GenericConfigLocation) +
        "/ok-input-method";
    const QString m_homeConfigFilePath =
        m_homeConfigFileDir + "/ok-input-method.conf";
};

#endif // STATUSBARSETTINGS_H
